local questCompanionFrame = CreateFrame("Frame", "DLUQuestCompanionFrame");
local settingsFrame = CreateFrame("Frame", "DLUSettingsFrame");

local mapIds = {};
local legionMapIds = {1014, 1015, 1017, 1018, 1021, 1024, 1033};
DLU.tableMerge(mapIds, legionMapIds);

local questTable = {};
questTable[1014] = {};
questTable[1015] = {};
questTable[1017] = {};
questTable[1018] = {};
questTable[1021] = {};
questTable[1024] = {};
questTable[1033] = {};

local showMessage = true;
local lastRefreshHour, lastRefreshMinute = nil;

local WQIcons = {};
WQIcons[1] = "Interface\\ICONS\\Trade_Engineering";
WQIcons[3] = "Interface\\PVPFrame\\Icons\\prestige-icon-3";
WQIcons[4] = "Interface\\ICONS\\Pet_Type_Beast";
WQIcons[6] = "Interface\\MINIMAP\\Dungeon";
WQIcons[7] = "Interface\\MINIMAP\\TempleofKotmogu_ball_green";
WQIcons[8] = "Interface\\MINIMAP\\Raid";

-- Globals
function DLU.WorldQuestCheck()
	if (DLUSettings.WQCompanion) then
		local currentMapId = GetCurrentMapAreaID();
		
		if (currentMapId == 1007) then
			if (showMessage) then
				DLU.createMessage("Loading World Quests...");
			end
			
			for i = 1, #legionMapIds do
				if (legionMapIds[i] ~= nil) then
					local zoneId = legionMapIds[i];
					local taskInfo = C_TaskQuest.GetQuestsForPlayerByMapID(zoneId);
					if (taskInfo ~= nil) then
						for i, info in ipairs(taskInfo) do
							DLU.WorldQuestInsert(zoneId, info.questId);
						end
					end
				end
			end
		end
		
		if (showMessage) then
			for i = 1, #mapIds do
				local zoneId = mapIds[i];
				local questInfo = questTable[zoneId];
				local questAmount = #questInfo;
				local zoneName = GetMapNameByID(mapIds[i]);
				if (questAmount > 0) then
					DLU.createMessage(format("%i %s World Quests loaded...", questAmount, zoneName));
				end
			end
		end
		
		showMessage = false;
		DLU.WorldQuestCompleteCheck();
		
		local currentGameTimeHour, currentGameTimeMinute = GetGameTime();
		if (lastRefreshHour ~= currentGameTimeHour and lastRefreshMinute == currentGameTimeMinute) then
			DLU.WorldQuestRefreshTable();
		end
	end
end

function DLU.WorldQuestDetails()
	for i = 1, #mapIds do
		local zoneId = mapIds[i];
		local zoneName = GetMapNameByID(zoneId);
		local questInfo = questTable[zoneId];
		local questAmount = #questInfo;
		
		if (questAmount > 0) then
			DLU.createMessage(format("%i World Quests in %s:", #questInfo, zoneName));
			for k, v in pairs(questInfo) do
				local questTitle = DLU.getWQRarityTitle(v);
				print(format("%s", questTitle));
			end
		end
	end
end

function DLU.WorldQuestCompleteCheck()
	for i = 1, #mapIds do
		local zoneId = mapIds[i];
		local questInfo = questTable[zoneId];
		for k, v in pairs(questInfo) do
			local isCompleted = IsQuestFlaggedCompleted(v);
			if (isCompleted) then
				local questTitle = DLU.getWQRarityTitle(v);
				
				local message = format("%s completed!", questTitle);
				DLU.createMessage(message);
				if (DLUSettings.WQCompanionPartyNotification) then
					DLU.notifyParty(message);
				end
				table.remove(questInfo, k);
			end
		end
	end
end

function DLU.WorldQuestInsert(zoneId, questId)
	if (DLU.tableContains(questTable[zoneId], questId) == false) then
		table.insert(questTable[zoneId], questId);
		
		local _, _, worldQuestType = GetQuestTagInfo(questId);
		if (worldQuestType == LE_QUEST_TAG_TYPE_INVASION) then
			local zoneName = GetMapNameByID(zoneId);
			DLU.createMessage(format("Legion invasion detected in %s", zoneName));
		end
	end
end

function DLU.WorldQuestRefreshTable()
	DLU.createMessage("Refreshing World Quests");
	
	lastRefreshHour, lastRefreshMinute = GetGameTime();
	
	for i = 1, #mapIds do
		local zoneId = mapIds[i];
		for k in pairs(questTable[zoneId]) do
			questTable[zoneId][k] = nil;
		end
	end
	
	DLU.WorldQuestCheck();
end

function DLU.getWQRarityTitle(questId)
	local title, factionID = C_TaskQuest.GetQuestInfoByQuestID(questId);
	local title = DLU.getWQRarity(questId, title);
	
	return title;
end

function DLU.getWQRarity(questId, title)
	local _, _, WQType, rarity, isElite, tradeskillLineIndex = GetQuestTagInfo(questId);
	
	--[[
		World Quest types:
		1 = prof
		2 = normal
		3 = PvP
		4 = Pet Battle
		6 = Dungeon (Interface\MINIMAP\Dungeon)
		7 = Legion Invasion
		8 = Raid (Interface\MINIMAP\Raid)
		
		Maybe = (Interface\QuestTypeIcons)
		Elite (Interface\Tooltips\EliteNameplateIcon)
		Rare Elite (Interface\Tooltips\RareEliteNameplateIcon)
		
		Tradeskills:
		7 = Cooking
		8 = Fishing
	]]
	
	if (WQType == LE_QUEST_TAG_TYPE_RAID or WQType == LE_QUEST_TAG_TYPE_DUNGEON or WQType == LE_QUEST_TAG_TYPE_PVP or WQType == LE_QUEST_TAG_TYPE_PROFESSION or WQType == LE_QUEST_TAG_TYPE_PET_BATTLE or WQType == LE_QUEST_TAG_TYPE_INVASION) then
		title = format("%s%s", DLU.iconToText(WQIcons[WQType]), title);
	elseif (isElite) then
		local eliteIcon = "Interface\\Tooltips\\EliteNameplateIcon";
		title = format("%s%s", DLU.iconToText(eliteIcon), title);
	end
	
	local result = rarity;
	if (WQType == 8) then
		result = 4;
	elseif (rarity == 2) then
		result = 3;
	end
	
	title = DLU.itemColorString(result, title);
	
	return title;
end

function DLU.getRefreshTime()
	return format("%i:%i", lastRefreshHour, lastRefreshMinute);
end

settingsFrame:RegisterEvent("PLAYER_LOGIN");
settingsFrame:SetScript("OnEvent", function()
	DLU.getSettings();
	
	if (DLUSettings.WQCompanion) then
		lastRefreshHour, lastRefreshMinute = GetGameTime();
		questCompanionFrame:RegisterEvent("WORLD_MAP_UPDATE");
		questCompanionFrame:SetScript("OnEvent", DLU.WorldQuestCheck);
	end
end);